@@ -78,6 +78,31 @@ def search_for_bug(module):
                     continue
         return result
 
+def search_in_new(module):
+    if isinstance(module, _Npm2Deb):
+        module = module.name
+    my_print('Looking for packages in NEW:')
+    _debug(1, "calling new-check")
+    found = False
+    formatted = "  {0:20} {1:>3}"
+    url = "https://api.ftp-master.debian.org/sources_in_suite/new"
+    _debug(1, "opening url %s" % url)
+    data = _urlopen(url).read().decode('utf-8')
+    data = _parseJSON(data)
+    result = []
+    for package in data:
+        name = package['source']
+        version = package['version']
+        if not module in name:
+            continue
+        found = True
+        result.append(package)
+        my_print(formatted.format(package['source'],
+                                  package['version']
+                                 ))
+    if not found:
+        my_print("  None")
+    return result
 
 def search_for_reverse_dependencies(module):
     if isinstance(module, _Npm2Deb):
